#include "imageview.h"
#include "QPainter"
#include "QWidget"
#include "QPixmap"
#include "QTimeLine"
#include "QMouseEvent"
#include "QGuiApplication"
#include "QScreen"
#include "QtSvg"
#include <QDebug>
#include "dataManager/LangTransManager.cpp"

ImageView::ImageView(QWidget* parent , Qt::WindowFlags f):QWidget(parent,f){
    _bgColor = Qt::transparent;
}

void ImageView::setTag(void * tag){
    this->_tag = tag;
}

void * ImageView::getTag(){
    return this->_tag;
}

void ImageView::setRadius(int radius){
    this->_radius = radius;
    update();
}

void ImageView::setSign(QString sign){
    this->sign = sign;
}

QString ImageView::getSign(){
    return this->sign;
}

void ImageView::paintEvent(QPaintEvent * e){
    QPainter painter(this);
     painter.setRenderHint(QPainter ::Antialiasing);
    if(_clipRadius > 0){
        QPainterPath path;
        path.addRoundedRect(rect(),_clipRadius,_clipRadius);
        painter.setClipPath(path);
    }

    this->drawBackgroudColor(&painter);
    this->drawImage();
    this->drawWave();
    this->drawText();

}

void ImageView::drawBackgroudColor(QPainter * painter){
    if(_bgColor != Qt::transparent){
        painter->fillRect(rect(),_drawClickColor != nullptr ? _drawClickColor : _bgColor);
    }
}

void ImageView::setClickListener(ImageVClickListener clickListener){
    this->_clickListener = clickListener;
}

void ImageView::setOnClickListener(ImageClickListener* onClick){
    _onClick = onClick;
}

void ImageView::setClickable(bool clickable){
    this->_clickable = clickable;
}

void ImageView::drawWave(){
    if(_frame > 0 && _clickable){
        QPainter painter(this);

        QPainterPath path;
        if(_radius > 0){
            path.addRoundedRect(0,0,width(),height(),_radius,_radius);
            painter.setClipPath(path);
        }
        painter.setPen(Qt::NoPen);
        painter.fillRect(rect(),Qt::transparent);
        painter.setRenderHint(QPainter::Antialiasing);
        painter.setBrush(QColor(0,0,0,15));
        painter.drawEllipse(_downX -_frame ,_downY - _frame , _frame * 2 , _frame * 2);

    }
}

void ImageView::drawImage(){
    if(_map!= nullptr){
        const int w = width();
        const int h = height();
        const qreal scaleW = (_mapWidth ) * 1.0 / (w - _paddingLeft - _paddingRight);
        const qreal scaleH = (_mapHeight ) * 1.0 / (h - _paddingTop - _paddingBottom);

        qreal maxScale;

        isWMax = scaleW > scaleH;
        if(isWMax){
            maxScale = scaleW;
        }else{
            maxScale = scaleH;
        }

        QPainter painter(this);
        painter.setRenderHint(QPainter::SmoothPixmapTransform);

        int st = _imv_scaleType;
        painter.setPen(Qt::NoPen);
        if(st == incenter){
            if(scaleW <= 1 && scaleH <= 1){
                painter.drawPixmap(qMax((w - _mapWidth) / 2 , _paddingLeft),qMax((h - _mapHeight) / 2 , _paddingRight) , *_map);
             }else{
                st = isWMax ? fillWidth : fillHeight;
            }
        }

        if(st == fillWidth){
            qreal s = 1.0 / scaleW;
            int t = (h * scaleW - _mapHeight) / 2;
            int l = 0;
            if(t < 0)
                painter.setClipRect(0,0,w,h);
            painter.save();
            painter.scale(s,s);
            if(t < 0){
                painter.translate(0,t);
                t = 0;
            }
            painter.drawPixmap(qMax(l*1.0,_paddingLeft*scaleW),qMax(t*1.0,_paddingTop*scaleW),*_map);
            painter.restore();
        }else if(st == fillHeight){
            qreal s = 1.0 / scaleH;
            int t = 0;
            int l = (w*scaleH - _mapWidth ) / 2;
            if(l < 0)
                painter.setClipRect(0,0,w,h);
            painter.save();
            painter.scale(s,s);
            if(l < 0){
                painter.translate(l,0);
                l = 0;
            }

            painter.drawPixmap(qMax(l*1.0,_paddingLeft*scaleH),qMax(t*1.0,_paddingTop*scaleH),*_map);
            painter.restore();
        }else if(st == fillWH){
            painter.save();
            painter.scale(1.0 / scaleW,1.0 / scaleH);
            painter.drawPixmap(_paddingLeft*scaleW,_paddingTop*scaleH,*_map);
            painter.restore();
        }
    }
}

void ImageView::setBackgroudColor(QColor backgroudColor){
    this->_bgColor = backgroudColor;
    update();
}

void ImageView::setFontColor(QColor fontColor){
    this->_fontColor = fontColor;
    update();
}

void ImageView::setClipRadius(qreal radius){
    this->_clipRadius = radius;
}

void ImageView::changeEvent(QEvent * e){
    if(e->type() == QEvent::LanguageChange && _trKey.length() > 0){
          setLabel(QObject::tr(_trKey.c_str()),_trKey);
    }
}

void ImageView::drawText(){
    if(mText.size() == 0)return;
    QPainter p(this);
    p.setPen(_fontColor);
    p.setRenderHint(QPainter::Antialiasing);
    p.setFont(_font);
    this->measureText(&p);//手动测量，减少系统每次重回都要重新测量

    int l = (width() - _textSize.width()) / 2;
    int t = (height() + _textSize.height()) / 2;

    p.drawText( qMax(l,_paddingLeft),qMax(t,_paddingTop) , mText);//减少重新测量的耗时

}

void ImageView::measureText(QPainter* p){
    if(_reMeasureTextSize){
       if(!p->isActive())return;
       QFontMetrics fm = p->fontMetrics();
       int width = fm.width(mText);//获得一个字符串的宽度
       int height = fm.ascent() - fm.descent() ;
       _textSize.setWidth(width);
       _textSize.setHeight(height);
       _reMeasureTextSize = false;
    }
}

void ImageView::waveCenter(bool center){
    _waveCenter = center;
}

void ImageView::setLabelFromTrKey(const string& trKey){
    string key = (string)trKey;
    setLabel(QObject::tr(key.c_str()),key);
}

void ImageView::setLabel(QString text,const string& trKey ){
    if(text.isNull()){
        text = "";
    }
    this->_trKey = (string)trKey;
    if(mText.size() > 0 && mText == text)return;
    _reMeasureTextSize = true;
    mText = text;
    update();
 }

void ImageView::setLabelFont(QFont font){
    this->_font = font;
    _reMeasureTextSize = true;
    update();
}

void ImageView::changeBgColorByAnim(QColor startColor,QColor endColor){
    if(_bgColor == nullptr)return;
    if(_pressTimerLine != nullptr &&_timerLine->state() == _timerLine->Running){
        _pressTimerLine->stop();

        startColor = _drawClickColor;
    }
    _pressTimerLine = new QTimeLine(130,this);
    _pressTimerLine->setFrameRange(0,100);
    connect(_pressTimerLine,&QTimeLine::frameChanged,this,[=](int frame){
        if(_drawClickColor == nullptr){
            _drawClickColor = QColor();
        }
        int bcr = startColor.red();
        int bcg = startColor.green();
        int bcb = startColor.blue();
        int bca = startColor.alpha();

        qreal percent = frame / 100.0;

        _drawClickColor.setRgba(qRgba(
                                    bcr+ percent * (endColor.red() - bcr),
                                    bcg+ percent * (endColor.green() - bcg),
                                    bcb+ percent * (endColor.blue() - bcb),
                                    bca+ percent * (endColor.alpha() - bca)
                                      )
                                );
        update();
    } );
    _pressTimerLine->start();
}

bool ImageView::existBgColr(){
    return _bgColor != nullptr && _bgColor !=Qt::transparent;
}

void ImageView::mousePressEvent(QMouseEvent *e){
    emit mouseEvent(1);
    _beClicked = true;
    _isMousePressed = true;
    if(!_clickable)return;

    _drawClickColor = nullptr;
    QColor drawClickColr = nullptr;
    if(existBgColr()){
        if(_clickColor != nullptr){
            drawClickColr = _clickColor;
        }else{
            if(_bgColor.alpha() < 160){
                drawClickColr = QColor(_bgColor);
                drawClickColr.setAlpha(_bgColor.alpha() + 95);
            }else{
                drawClickColr = QColor(_bgColor.red(),_bgColor.green(),_bgColor.blue(),_bgColor.alpha());
            }
        }
        changeBgColorByAnim(_bgColor,drawClickColr);
    }


    if(_timerLine!= nullptr &&_timerLine->state() == _timerLine->Running){
        return;
    }
    _timerLine = new QTimeLine(1500,this);

    if(_waveCenter){
        _downX = width() / 2;
        _downY = height() / 2;
    }else{
        _downX = e->x();
        _downY = e->y();
    }

    _timerLine->setFrameRange(10,width());
    connect(_timerLine,&QTimeLine::frameChanged,this,[=](int frame){
        _frame = frame;

        if(existBgColr() && !_beClicked && (_pressTimerLine == nullptr || _pressTimerLine->state() != _pressTimerLine->Running)){
            _timerLine->stop();
             _frame = 0;
             update();
             return;
        }

        if(_frame == _timerLine->endFrame()){
            if(!_mouseEnter || !_isMousePressed){
                _frame = 0;
            }
         }
        update();
    } );
    _timerLine->start();
}

void ImageView::setClickColor(QColor clickColor){
    this->_clickColor = clickColor;
    update();
}

void ImageView::mouseReleaseEvent(QMouseEvent * e) {
    emit mouseEvent(3);
    _beClicked = false;
    _isMousePressed = false;
    if(!_clickable)return;
    if(!existBgColr()){
        if( _timerLine != nullptr && _timerLine->state() == _timerLine->Running){
            _timerLine->stop();
            _timerLine->setFrameRange(_frame,width());
            _timerLine->setDuration(89*(width() - _frame) / width());
            _timerLine->start();
        }else{
            _frame = 0;
            update();
        }
    }
    if(_drawClickColor == nullptr){
        _drawClickColor = _bgColor;
    }
    changeBgColorByAnim(_drawClickColor,_bgColor);
    int x = e->x();
    int y = e->y();
    if(/*_mouseEnter && */(x  > 0 && x < width() && y > 0 && y < height())){
        if(_clickListener){
            _clickListener(this,e);
        }
        emit clicked();
        if(_onClick){
            _onClick->onClick(this,e);
        }
    }
  }

void ImageView::mouseMoveEvent(QMouseEvent *) {
    emit mouseEvent(2);
}

void ImageView::leaveEvent(QEvent *) {
    _mouseEnter = false;
    if(_timerLine != nullptr && _timerLine->state() != _timerLine->Running){
        _frame = 0;
        update();
    }
    emit mouseEvent(-1);
}

void ImageView::enterEvent(QEvent *) {
    _mouseEnter = true;
    emit mouseEvent(0);
}

void ImageView::createNewPixmap(){
    if(_src_map.isNull() || _src_map.isDetached())return;
    if(_map != nullptr){
        delete _map;
        _map = nullptr;
    }
    double pr = 1;
    if(width() > _src_map.width() && height() > _src_map.height()){
        _map = new QPixmap(_src_map);
    }else{
        pr = QGuiApplication::primaryScreen()->devicePixelRatio();
        qreal scale = qMin(width() * 1.0 / _src_map.width(),height() * 1.0 / _src_map.height());
        QPixmap p = _src_map.scaledToWidth(scale * _src_map.width() * pr,Qt::SmoothTransformation);
        p.setDevicePixelRatio(pr);
        _map = new QPixmap(p);
    }
    this->freshMapData(pr);
 }

void ImageView::freshMapData(double scale){
    _mapWidth = _map->width()/scale;
    _mapHeight = _map->height()/scale;
}

void ImageView::resizeEvent(QResizeEvent *) {
    _sizeInited = true;
   createNewPixmap();
}

void ImageView::setPixMapSrc(QPixmap map){
    _src_map = map;
    if(_sizeInited){
        createNewPixmap();
    }
 }

void ImageView::setSvgPixMapSrc(QString path,int width,int height){
    if(_map != nullptr){
        delete  _map;
        _map = nullptr;
    }

    QSvgRenderer* svgRender = new QSvgRenderer();
    svgRender->load(path);

    _map = new QPixmap(width*2,height*2);
    _map->fill(Qt::transparent);
    QPainter p(_map);
    svgRender->render(&p);

    freshMapData(1);
}

void ImageView::setScaleType(IMVScaleType scaleType){
    if(_imv_scaleType == scaleType)return;
    _imv_scaleType = scaleType;
    update();
}

void ImageView::setPadding(int left,int top,int right ,int bottom){
    this->_paddingLeft = left;
    this->_paddingTop = top;
    this->_paddingRight = right;
    this->_paddingBottom = bottom;
    update();
}

ImageView::~ImageView(){
    if(_map != nullptr){
        delete  _map;
        _map = nullptr;
    }
}


